// Copyright 2020-2022 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go-apiclientconnect. DO NOT EDIT.

package registryv1alpha1apiclientconnect

import (
	context "context"
	registryv1alpha1connect "github.com/bufbuild/buf/private/gen/proto/connect/buf/alpha/registry/v1alpha1/registryv1alpha1connect"
	v1alpha1 "github.com/bufbuild/buf/private/gen/proto/go/buf/alpha/registry/v1alpha1"
	connect_go "github.com/bufbuild/connect-go"
	zap "go.uber.org/zap"
)

type organizationServiceClient struct {
	logger          *zap.Logger
	client          registryv1alpha1connect.OrganizationServiceClient
	contextModifier func(context.Context) context.Context
}

// GetOrganization gets a organization by ID.
func (s *organizationServiceClient) GetOrganization(ctx context.Context, id string) (organization *v1alpha1.Organization, _ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	response, err := s.client.GetOrganization(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.GetOrganizationRequest{
				Id: id,
			}),
	)
	if err != nil {
		return nil, err
	}
	return response.Msg.Organization, nil
}

// GetOrganizationByName gets a organization by name.
func (s *organizationServiceClient) GetOrganizationByName(ctx context.Context, name string) (organization *v1alpha1.Organization, _ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	response, err := s.client.GetOrganizationByName(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.GetOrganizationByNameRequest{
				Name: name,
			}),
	)
	if err != nil {
		return nil, err
	}
	return response.Msg.Organization, nil
}

// ListOrganizations lists all organizations.
func (s *organizationServiceClient) ListOrganizations(
	ctx context.Context,
	pageSize uint32,
	pageToken string,
	reverse bool,
) (organizations []*v1alpha1.Organization, nextPageToken string, _ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	response, err := s.client.ListOrganizations(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.ListOrganizationsRequest{
				PageSize:  pageSize,
				PageToken: pageToken,
				Reverse:   reverse,
			}),
	)
	if err != nil {
		return nil, "", err
	}
	return response.Msg.Organizations, response.Msg.NextPageToken, nil
}

// ListUserOrganizations lists all organizations a user is member of.
func (s *organizationServiceClient) ListUserOrganizations(
	ctx context.Context,
	userId string,
	pageSize uint32,
	pageToken string,
	reverse bool,
) (organizations []*v1alpha1.OrganizationMembership, nextPageToken string, _ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	response, err := s.client.ListUserOrganizations(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.ListUserOrganizationsRequest{
				UserId:    userId,
				PageSize:  pageSize,
				PageToken: pageToken,
				Reverse:   reverse,
			}),
	)
	if err != nil {
		return nil, "", err
	}
	return response.Msg.Organizations, response.Msg.NextPageToken, nil
}

// CreateOrganization creates a new organization.
func (s *organizationServiceClient) CreateOrganization(ctx context.Context, name string) (organization *v1alpha1.Organization, _ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	response, err := s.client.CreateOrganization(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.CreateOrganizationRequest{
				Name: name,
			}),
	)
	if err != nil {
		return nil, err
	}
	return response.Msg.Organization, nil
}

// DeleteOrganization deletes a organization.
func (s *organizationServiceClient) DeleteOrganization(ctx context.Context, id string) (_ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	_, err := s.client.DeleteOrganization(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.DeleteOrganizationRequest{
				Id: id,
			}),
	)
	if err != nil {
		return err
	}
	return nil
}

// DeleteOrganizationByName deletes a organization by name.
func (s *organizationServiceClient) DeleteOrganizationByName(ctx context.Context, name string) (_ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	_, err := s.client.DeleteOrganizationByName(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.DeleteOrganizationByNameRequest{
				Name: name,
			}),
	)
	if err != nil {
		return err
	}
	return nil
}

// AddOrganizationMember add a role to an user in the organization.
func (s *organizationServiceClient) AddOrganizationMember(
	ctx context.Context,
	organizationId string,
	userId string,
	organizationRole v1alpha1.OrganizationRole,
) (_ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	_, err := s.client.AddOrganizationMember(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.AddOrganizationMemberRequest{
				OrganizationId:   organizationId,
				UserId:           userId,
				OrganizationRole: organizationRole,
			}),
	)
	if err != nil {
		return err
	}
	return nil
}

// UpdateOrganizationMember update the user's membership information in the organization.
func (s *organizationServiceClient) UpdateOrganizationMember(
	ctx context.Context,
	organizationId string,
	userId string,
	organizationRole v1alpha1.OrganizationRole,
) (_ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	_, err := s.client.UpdateOrganizationMember(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.UpdateOrganizationMemberRequest{
				OrganizationId:   organizationId,
				UserId:           userId,
				OrganizationRole: organizationRole,
			}),
	)
	if err != nil {
		return err
	}
	return nil
}

// RemoveOrganizationMember remove the role of an user in the organization.
func (s *organizationServiceClient) RemoveOrganizationMember(
	ctx context.Context,
	organizationId string,
	userId string,
) (_ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	_, err := s.client.RemoveOrganizationMember(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.RemoveOrganizationMemberRequest{
				OrganizationId: organizationId,
				UserId:         userId,
			}),
	)
	if err != nil {
		return err
	}
	return nil
}

// SetOrganizationMember sets the role of a user in the organization.
func (s *organizationServiceClient) SetOrganizationMember(
	ctx context.Context,
	organizationId string,
	userId string,
	organizationRole v1alpha1.OrganizationRole,
) (_ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	_, err := s.client.SetOrganizationMember(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.SetOrganizationMemberRequest{
				OrganizationId:   organizationId,
				UserId:           userId,
				OrganizationRole: organizationRole,
			}),
	)
	if err != nil {
		return err
	}
	return nil
}

// GetOrganizationSettings gets the settings of an organization, including organization base roles.
func (s *organizationServiceClient) GetOrganizationSettings(
	ctx context.Context,
	organizationId string,
) (repositoryBaseRole v1alpha1.RepositoryRole, pluginBaseRole v1alpha1.PluginRole, templateBaseRole v1alpha1.TemplateRole, membersCount uint32, _ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	response, err := s.client.GetOrganizationSettings(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.GetOrganizationSettingsRequest{
				OrganizationId: organizationId,
			}),
	)
	if err != nil {
		return v1alpha1.RepositoryRole(0), v1alpha1.PluginRole(0), v1alpha1.TemplateRole(0), 0, err
	}
	return response.Msg.RepositoryBaseRole, response.Msg.PluginBaseRole, response.Msg.TemplateBaseRole, response.Msg.MembersCount, nil
}

// UpdateOrganizationSettings update the organization settings including base roles.
func (s *organizationServiceClient) UpdateOrganizationSettings(
	ctx context.Context,
	organizationId string,
	repositoryBaseRole v1alpha1.RepositoryRole,
	pluginBaseRole v1alpha1.PluginRole,
	templateBaseRole v1alpha1.TemplateRole,
) (_ error) {
	if s.contextModifier != nil {
		ctx = s.contextModifier(ctx)
	}
	_, err := s.client.UpdateOrganizationSettings(
		ctx,
		connect_go.NewRequest(
			&v1alpha1.UpdateOrganizationSettingsRequest{
				OrganizationId:     organizationId,
				RepositoryBaseRole: repositoryBaseRole,
				PluginBaseRole:     pluginBaseRole,
				TemplateBaseRole:   templateBaseRole,
			}),
	)
	if err != nil {
		return err
	}
	return nil
}
